﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Warship
{
    internal class MapWithSetWarshipsGeneric<T,U>
        where T : class, IDrawingObject
        where U : AbstractMap
    {
        private readonly int _pictureWidth;
        private readonly int _pictureHeight;
        private readonly int _placeSizeWidth = 120;
        private readonly int _placeSizeHeight = 50;
        private readonly SetWarshipsGeneric<T> _setWarship;
        private readonly U _map;

        public MapWithSetWarshipsGeneric(int picWidth,int picHeight, U map)
        {
            int width = picWidth / _placeSizeWidth;
            int height = picHeight/_placeSizeHeight;
            _setWarship = new SetWarshipsGeneric<T>(width * height);
            _pictureWidth = picWidth;
            _pictureHeight = picHeight;
            _map = map;
        }

        public static int operator +(MapWithSetWarshipsGeneric<T,U> map, T warship)
        {
            return map._setWarship.Insert(warship);
        }

        public static T operator -(MapWithSetWarshipsGeneric<T,U> map, int position)
        {
            return map._setWarship.Remove(position);
        }

        public Bitmap ShowSet()
        {
            Bitmap bmp = new(_pictureWidth, _pictureWidth);
            Graphics gr = Graphics.FromImage(bmp);
            DrawBackground(gr);
            DrawWarship(gr);
            return bmp;
        }

        public Bitmap ShowOnMap()
        {
            Shaking();
            for (int i = 0; i < _setWarship.Count; i++)
            {
                var warship = _setWarship.Get(i);
                if (warship != null)
                {
                    return _map.CreateMap(_pictureWidth, _pictureHeight, warship);
                }
            }
            return new(_pictureWidth, _pictureHeight);
        }

        public Bitmap MoveObject(Direction direction)
        {
            if (_map != null)
            {
                return _map.MoveObject(direction);
            }
            return new(_pictureWidth, _pictureHeight);
        }

        public void Shaking()
        {
            int j = _setWarship.Count - 1;
            for (int i = 0; i < _setWarship.Count; i++)
            {
                if (_setWarship.Get(i) == null)
                {
                    for (; j > i; j--)
                    {
                        var warship = _setWarship.Get(j);
                        if (warship != null)
                        {
                            _setWarship.Insert(warship, i);
                            _setWarship.Remove(j);
                            break;
                        }
                    }
                    if (j <= i)
                    {
                        return;
                    }
                }
            }
        }

        private void DrawBackground(Graphics gr)
        {
            Pen pen = new(Color.Black, 5);
            for (int i = 0; i < _pictureWidth / _placeSizeWidth; i++)
            {
                for (int j = 0; j < _pictureHeight / _placeSizeHeight + 1; ++j)
                {
                    gr.DrawLine(pen, i * _placeSizeWidth + 20, j * _placeSizeHeight+2, i * _placeSizeWidth + (int)(_placeSizeWidth*0.8), j * _placeSizeHeight+2);
                    gr.DrawLine(pen, i * _placeSizeWidth + 20, j * _placeSizeHeight + _placeSizeHeight/2+2, i * _placeSizeWidth + (int)(_placeSizeWidth * 0.8), j * _placeSizeHeight + _placeSizeHeight/2+2);
                    gr.DrawLine(pen, i * _placeSizeWidth + (int)(_placeSizeWidth * 0.8), j * _placeSizeHeight + 2, i * _placeSizeWidth + _placeSizeWidth, j * _placeSizeHeight + _placeSizeHeight/2);
                    gr.DrawLine(pen, i * _placeSizeWidth + _placeSizeWidth, j * _placeSizeHeight + _placeSizeHeight / 2, i * _placeSizeWidth+ (int)(_placeSizeWidth * 0.8), j * _placeSizeHeight + _placeSizeHeight);
                }
            }
        }

        private void DrawWarship(Graphics gr)
        {
            int width = _pictureWidth / _placeSizeWidth;
            int height = _pictureHeight / _placeSizeHeight;

            for (int i = 0; i < _setWarship.Count; i++)
            {
                if (_setWarship.Get(i) != null)
                {
                    _setWarship.Get(i).SetObject(i % width * _placeSizeWidth, (height - 1 - i / width) * _placeSizeHeight, _pictureWidth, _pictureHeight);
                    _setWarship.Get(i)?.DrawingObject(gr);
                }
            }
        }
    }
}
